# -*- coding: utf-8 -*-
"""PyAdaptiveCards internal components.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import json

class Serializable:
    """Parent class for all components of adaptive cards.

    Each component should inherit from this class and then specify, from
    its properties, which fall into the following two categories:

    * Simple properties are text properties like "type" or "id"
    * Serializable properties are properties that can themselfes be serilized.
    This includes lists of items (i.e. the 'body' field of the adaptive card) or
    single objects that also inherit from Serializable
    """
    def __init__(self, serializable_properties, simple_properties):
        """Creates a serializable object.

        See class docstring for an explanation what the different types of
        properties are.

        Args:
            serializable_properties(list): List of all serializable properties
            simple_properties(list): List of all simple properties.
        """
        self.serializable_properties = serializable_properties
        self.simple_properties = simple_properties

    def to_json(self, pretty=False):
        """Create json from a serializable component

        This function is used to render the json from a component. While all
        components do support this operation it is mainly used on the
        AdaptiveCard to generate the json for the attachment.

        Args:
            pretty(boolean): If true, the returned json will be sorted by keys
                and indented with 4 spaces to make it more human-readable

        Returns:
            A Json representation of this component
        """
        ret = None
        if pretty:
            ret = json.dumps(self.to_dict(), indent=4, sort_keys=True)
        else:
            ret = json.dumps(self.to_dict())

        return ret

    def to_dict(self):
        """Export a dictionary representation of this card/component by
        parsing all simple and serializable properties.

        A simple_component is a single-text property of the exported card
        (i.e. {'version': "1.2"}) while a serializable property is another
        subcomponent that also implements a to_dict() method.

        The to_dict() method is used to recursively create a dict representation
        of the adaptive card. This dictionary representation can then be
        converted into json for usage with the API.

        Returns:
            dict: Dictionary representation of this component.
        """
        export = {}

        # Export simple properties (i.e. properties that are only single text)
        for sp in self.simple_properties:
            o = getattr(self, sp, None)

            if getattr(o, 'to_dict', None):
                export[sp] = o.to_dict()
            elif o is not None:
                export[sp] = o

        # Export all complex properties by calling its respective serialization
        for cp in self.serializable_properties:
            o = getattr(self, cp, None)

            if o is not None:
                # Check if it is a list or a single component
                l = []
                if isinstance(o, list):
                    for i in o:
                        l.append(i.to_dict())
                else:
                    l = o.to_dict()
                export[cp] = l

        return export



# -*- coding: utf-8 -*-
"""PyAdaptiveCards components wrapper.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .abstract_components import Serializable
from .utils import check_type
from .options import BlockElementHeight, Spacing, ImageSize, ImageStyle, Colors, FontSize, ContainerStyle, HorizontalAlignment
from .actions import OpenUrl, ShowCard, Submit

class MediaSource(Serializable):
    """Defines the source of a Media element."""
    def __init__(self,
                 mimeType,
                 url):
        """Create a new MediaSource component.

        Args:
            mimeType(str): Mime type of the associated media(i.e. 'video/mp4')
            url(str): URL of the media.
        """
        # Check types
        check_type(mimeType, str, False, False)
        check_type(url, str, False, False)

        self.mimeType = mimeType
        self.url = url

        super().__init__(serializable_properties=[],
                         simple_properties=['mimeType', 'url'])

class Media(Serializable):
    """Displays a media player for audio or video content"""
    def __init__(self,
                 sources,
                 poster=None,
                 altText=None,
                 height=None,
                 separator=None,
                 spacing=None,
                 id=None):
        """Create a new Media component.

        Args:
            sources(list): A list of media sources to be played
            poster(str): The url to the image that is displayed before playing
            altText(str): Alternative text for this component
            height(BlockElementHeight): The height of this block element
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
            id(str): The id of this component
        """
        # Check types
        check_type(sources, MediaSource, True, False)
        check_type(poster, str, False, True)
        check_type(altText, str, False, True)
        check_type(height, BlockElementHeight, False, True)
        check_type(separator, bool, False, True)
        check_type(spacing, Spacing, False, True)
        check_type(id, str, False, True)

        self.type = "Media"
        self.sources = sources #Needs to be a list of media sources
        self.poster = poster
        self.altText = altText
        self.height = height
        self.separator = separator
        self.spacing = spacing
        self.id = id

        super().__init__(serializable_properties=['sources'],
                         simple_properties=[
                            'type', 'poster', 'altText', 'height',
                            'separator', 'spacing', 'id'
                         ])
class Image(Serializable):
    """Displays a image object"""

    def __init__(self,
                 url,
                 altText=None,
                 backgroundColor=None,
                 height=None,
                 horizontalAlignment=None,
                 selectAction=None,
                 size=None,
                 style=None,
                 width=None,
                 separator=None,
                 spacing=None,
                 id=None):
        """Create a new image component

        Args:
            url(str): The URL to the image
            altText(str): Alternative text describing the image
            backgroundColor(str): Background color for transparent images.
            height(str, BlockElementHeight): Height of the image either as a
                pixel value(i.e. '50px') or as an instance of BlockElementHeight
            horizontalAlignmnet(HorizontalAlignment): Controls how the component
                is positioned within its parent.
            selectAction(OpenUrl, Submit): Option that is caried out when the
                card is selected.
            size(ImageSize): Controls the approximate size of the image.
            style(ImageStyle): The display style of this image.
            width(str): Width of the image as a pixel value (i.e. '50px')
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
            id(str): The id of this component

        """
        check_type(url, str, False, False)
        check_type(altText, str, False, True)
        check_type(backgroundColor, str, False, True)
        check_type(height, (str, BlockElementHeight), False, True)
        check_type(horizontalAlignment, HorizontalAlignment, False, True)
        check_type(selectAction, (OpenUrl, Submit), False, True)
        check_type(size, ImageSize, False, True)
        check_type(style, ImageStyle, False, True)
        check_type(width, str, False, True)
        check_type(separator, bool, False, True)
        check_type(spacing, Spacing, False, True)
        check_type(id, str, False, True)

        self.type = "Image"
        self.url = url
        self.altText = altText
        self.backgroundColor = backgroundColor
        self.height = height
        self.horizontalAlignment = horizontalAlignment
        self.selectAction = selectAction
        self.size = size
        self.style = style
        self.width = width
        self.separator = separator
        self.spacing = spacing
        self.id = id

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'type', 'url', 'altText', 'backgroundColor',
                            'height', 'horizontalAlignment', 'selectAction',
                            'size', 'style', 'width', 'separator', 'spacing',
                            'id'
                         ])
class TextBlock(Serializable):
    """Component that contains text."""
    def __init__(self,
                 text,
                 color=None,
                 horizontalAlignment=None,
                 isSubtle=None,
                 maxLines=None,
                 size=None,
                 weight=None,
                 wrap=None,
                 separator=None,
                 spacing=None,
                 id=None):
        """Create a new TextBlock component.

        Args:
            text(str): The text to be displayed.
            color(Colors): The color of the text.
            horizontalAlignment(HorizontalAlignment): Controls how the component
                is positioned within its parent.
            isSubtle(bool): If true, displays text slightly toned down to appear 
                less prominent.
            maxLines(int): Specifies the number of lines to display.
            size(FontSize): Controls the size of the text.
            weight(FontWeight): Controls the weight of a TextBlock element.
            wrap(bool): If true, allow text to wrap.
            separator(bool): Draw a separating line when set to true.
            spacing(Spacing): Specify the spacing of this component.
            id(str): The id of this component.
        """


        #ToDo(mneiding): Type check
        check_type(text, str, False, False)
        check_type(color, Colors, False, True)

        self.type = "TextBlock"
        self.text = text
        self.color = color
        self.horizontalAlignment = horizontalAlignment
        self.isSubtle = isSubtle
        self.maxLines = maxLines
        self.size = size
        self.weight = weight
        self.wrap = wrap
        self.separator = separator
        self.spacing = spacing
        self.id = id

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'type', 'text', 'color', 'horizontalAlignment',
                            'isSubtle', 'maxLines', 'size', 'weight', 'wrap',
                            'spacing', 'id', 'separator'
                        ])

class Column(Serializable):
    """A column contains other components and (together with a ColumnSet) allows to display 
        multiple components under each other. 
    """
    def __init__(self, items=None,
                       separator=None,
                       spacing=None,
                       selectAction=None,
                       style=None,
                       verticalContentAlignment=None,
                       width=None,
                       id=None):
        """Create a new column.

        Args:
            items(list): List of components in this column
            separator(bool): Draw a separating line when set to true.
            spacing(Spacing): Specify the spacing of this component.
            selectAction(OpenUrl, Submit): Specifies the action that is invoked when the colum is
                selected.
            style(ContainerStyle): The style of this column.
            verticalContentAlignment(VerticalContentAlignment): The vertical alignment of the column
                content.
            width(str, int): The width of this column. Can be "auto", "stretch" or a number that 
                represents the width in pixels.
            id(str): The id of this component. 
        """
        self.type = "Column"
        self.items = items
        self.separator = separator
        self.spacing = spacing
        self.selectAction = selectAction
        self.style = style
        self.verticalContentAlignment = verticalContentAlignment
        self.width = width
        self.id = id

        super().__init__(serializable_properties=['items'],
                         simple_properties=[
                            'type', 'separator', 'spacing', 'selectAction',
                            'style', 'verticalContentAlignment', 'width', 'id'
                         ])

class Fact(Serializable):
    """Fact component to display a set of facts. 
    """
    def __init__(self, title, value):
        """Create a new Fact.
        
        Args:
            title(str): The title of this fact
            value(str): The value of this fact
        """
        self.title = title
        self.value = value

        super().__init__(serializable_properties=[],
                         simple_properties=['title', 'value'])

class Choice(Serializable):
    """Choice component. 

    Can be used together with a ChoiceSet to display different choices. 
    """
    def __init__(self, title, value):
        """Create a new choice. 

        Args:
            title(str): The title of the choice
            value(str): The value of the choice
        """
        self.title = title
        self.value = value

        super().__init__(serializable_properties=[],
                         simple_properties=['title', 'value'])



# -*- coding: utf-8 -*-
"""PyAdaptiveCards container wrapper.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .abstract_components import Serializable
from .options import ImageSize

class Container(Serializable):
   """Groups items together."""
   def __init__(self, items, selectAction=None,
                             style=None,
                             verticalContentAlignment=None,
                             height=None,
                             separator=None,
                             spacing=None,
                             id=None):
      """Create a new container.

      Args:
         items(list): List of items this container should contain.
         selectAction(OpenURL, Submit): Action to be invoked when container is selected.
         style(ContainerStyle): Style of this container.
         verticalContentAlignment(VerticalContentAlignment): Specifies vertical alignment 
            of the content. 
         height(BlockElementHeight): Specifies the way the height of this container should 
            be calculated (stretch or auto).
         separator(bool): Draw a separating line when set to true
         spacing(Spacing): Specify the spacing of this component
         id(str): The id of this component
      """
      self.type = "Container"
      self.items = items
      self.selectAction = selectAction
      self.style = style
      self.verticalContentAlignment = verticalContentAlignment
      self.height = height
      self.separator = separator
      self.spacing = spacing
      self.id = id

      super().__init__(serializable_properties=['items'],
                       simple_properties=[
                           'selectAction', 'style', 'verticalContentAlignment',
                           'height', 'separator', 'spacing', 'id', 'type'
                       ])

class ColumnSet(Serializable):
   """Set of columns to display content side by side."""
   def __init__(self, columns=None,
                      selectAction=None,
                      height=None,
                      separator=None,
                      spacing=None,
                      id=None):
      """Create a new ColumnSet.

      Args:
         columns(list): List of Column elements.
         selectAction(OpenURL, Submit): Action to be invoked when container is selected.
         height(BlockElementHeight): Specifies the way the height of this container should 
            be calculated (stretch or auto).
         separator(bool): Draw a separating line when set to true
         spacing(Spacing): Specify the spacing of this component
         id(str): The id of this component
      """
      self.type = "ColumnSet"
      self.columns = columns
      self.selectAction = selectAction
      self.height = height
      self.separator = separator
      self.spacing = spacing
      self.id = id

      super().__init__(serializable_properties=['columns', 'selectAction'],
                       simple_properties=[
                        'height', 'separator', 'spacing',
                        'id', 'type'
                      ])

class FactSet(Serializable):
   """Set of facts to create a factoide."""
   def __init__(self, facts, height=None,
                             separator=None,
                             spacing=None,
                             id=None):
      """Create a new FactSet.

      Args:
         facts(list): List of Fact Elements to be displayed.
         height(BlockElementHeight): Specifies the way the height of this container should 
            be calculated (stretch or auto).
         separator(bool): Draw a separating line when set to true
         spacing(Spacing): Specify the spacing of this component
         id(str): The id of this component
      """
      self.type = "FactSet"
      self.facts = facts
      self.height = height
      self.separator = separator
      self.spacing = spacing
      self.id = id

      super().__init__(serializable_properties=['facts'],
                       simple_properties=[
                           'type', 'height', 'separator', 'id', 'spacing'
                       ])

class ImageSet(Serializable):
   def __init__(self, images, imageSize=None,
                              height=None,
                              separator=None,
                              spacing=None,
                              id=None):
      """Create a new ImageSet.
      
      Args:
         images(list): List of Image objects to be displayed in a image set.
         imageSize(ImageSize): Size of the image.
         height(BlockElementHeight): Specifies the way the height of this container should 
            be calculated (stretch or auto).
         separator(bool): Draw a separating line when set to true
         spacing(Spacing): Specify the spacing of this component
         id(str): The id of this component
      """
      
      self.type = "ImageSet"
      self.images = images
      self.imageSize = imageSize
      self.height = height
      self.separator = separator
      self.spacing = spacing
      self.id = id

      super().__init__(serializable_properties=['images'],
                       simple_properties=[
                           'imageSize', 'height', 'separator', 'spacing', 'id',
                           'type'
                      ])



# -*- coding: utf-8 -*-
"""PyAdaptiveCards options wrapper.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from enum import Enum

class AbstractOption(Enum):
    def to_value(self):
        return str(self.name).lower()

    def __str__(self):
        return self.to_value()

    def __repr__(self):
        return self.to_value()

    def to_dict(self):
        return self.to_value()

class VerticalContentAlignment:
    """Specifies the vertical alignment of a component or of components in a 
    container.
    """
    TOP = 1
    CENTER = 2
    BOTTOM = 3

class Colors(AbstractOption):
    """Specifies the color of a textblock"""
    DEFAULT = 1
    DARK = 2
    LIGHT = 3
    ACCENT = 4
    GOOD = 5
    WARNING = 6
    ATTENTION = 7

class HorizontalAlignment(AbstractOption):
    """Specifies the horizontal alignment of a component"""
    LEFT = 1
    CENTER = 2
    RIGHT = 3

class FontSize(AbstractOption):
    """Specifies the font size of a TextBlock"""
    DEFAULT = 1
    SMALL = 2
    MEDIUM = 3
    LARGE = 4
    EXTRALARGE = 5

class FontWeight(AbstractOption):
    """Specifies the font weight of a TextBlock"""
    DEFAULT = 1
    LIGHTER = 2
    BOLDER = 3

class BlockElementHeight(AbstractOption):
    """Specifies the way the height of a element is determined. """
    AUTO = 1
    STRETCH = 2

class Spacing(AbstractOption):
    """Specify the spacing around a component"""
    DEFAULT = 1
    NONE = 2
    SMALL = 3
    MEDIUM = 4
    LARGE = 5
    EXTRALARGE = 6
    PADDING = 7

class ImageSize(AbstractOption):
    """Specify the size scaling of a Image"""
    AUTO = 1
    STRETCH = 2
    SMALL = 3
    MEDIUM = 4
    LARGE = 5

class ImageStyle(AbstractOption):
    """Specifies the style of the image. 

    PERSON will make the picture rounded
    """
    DEFAULT = 1
    PERSON = 2

class ContainerStyle(AbstractOption):
    """Specifies the style of the container"""
    DEFAULT = 1
    EMPHASIS = 2

class TextInputStyle(AbstractOption):
    """Specifies the type of input that a Text input can expect"""
    TEXT = 1
    TEL = 2
    URL = 3
    EMAIL = 4

class ChoiceInputStyle(AbstractOption):
    """Specifies the display style for a choice input"""
    COMPACT = 1
    EXPANDED = 2



# -*- coding: utf-8 -*-
"""PyAdaptiveCards inputs wrapper.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .abstract_components import Serializable

class Text(Serializable):
    """Input field that accepts text."""
    def __init__(self, id, isMultiline=None,
                           maxLength=None,
                           placeholder=None,
                           style=None,
                           value=None,
                           height=None,
                           separator=None,
                           spacing=None):
        """Create a new text input.

        Args:
            id(str): The id of this input.
            isMultiline(bool): If True, multline content is allowed.
            maxLength(int): Hint for maximum number of characters (some clients ignore this).
            placeholder(str): Placeholder text to be displayed in this input field.
            style(TextInputStyle): Style of the text input (i.e. are we expecting a mail or a url)
            value(str): Initial value of this field.
            height(BlockElementHeight): Specifies the way the height of this container should 
                be calculated (stretch or auto).
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
        """

        self.type = "Input.Text"
        self.id = id
        self.isMultiline = isMultiline
        self.maxLength = maxLength
        self.placeholder = placeholder
        self.style = style
        self.value = value
        self.height = height
        self.separator = separator
        self.spacing = spacing

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'id', 'type', 'isMultiline', 'maxLength',
                            'placeholder', 'style', 'value', 'height',
                            'separator', 'spacing'
                        ])

class Number(Serializable):
    """Input field that accepts numbers."""
    def __init__(self, id, max=None,
                           min=None,
                           placeholder=None,
                           value=None,
                           height=None,
                           separator=None,
                           spacing=None):
        """Create a new number input.

        Args:
            id(str): The id of this input.
            max(int): The maximum accepted value. 
            min(int): The minimum accepted value.
            placeholder(str): Placeholder text to be displayed in this input field.
            value(int): Initial value of this field.
            height(BlockElementHeight): Specifies the way the height of this container should 
                be calculated (stretch or auto).
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
        """
        self.type = "Input.Number"
        self.id = id
        self.max = max
        self.min = min
        self.placeholder = placeholder
        self.value = value
        self.height = height
        self.separator = separator
        self.spacing = spacing

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'type', 'id', 'max', 'min', 'placeholder', 'value',
                            'height', 'separator', 'spacing'
                         ])

class Date(Serializable):
    """Input field that accepts date inputs."""
    def __init__(self, id, max=None,
                           min=None,
                           placeholder=None,
                           value=None,
                           height=None,
                           separator=None,
                           spacing=None):
        """Create a new Date.

        Args:
            id(str): The id of this input.
            max(str): The maximum date in ISO-8601 format.
            min(str): The minimum date in ISO-8601 format.
            placeholder(str): Placeholder text to be displayed in this input field.
            value(str): Initial value of this field.
            height(BlockElementHeight): Specifies the way the height of this container should 
                be calculated (stretch or auto).
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
        """
        self.type = "Input.Date"
        self.id = id
        self.max = max
        self.min = min
        self.placeholder = placeholder
        self.value = value
        self.height = height
        self.separator = separator
        self.spacing = spacing

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'type', 'id', 'max', 'min', 'placeholder', 'value',
                            'height', 'separator', 'spacing'
                         ])
class Time(Serializable):
    """Input field that accepts time inputs."""
    def __init__(self, id, max=None,
                           min=None,
                           placeholder=None,
                           value=None,
                           height=None,
                           separator=None,
                           spacing=None):
        """Create a new time input.
        
        Args:
            id(str): The id of this input. 
            max(str): Maximum value for this time field. Might be ignored by the client.
            min(str): Minimum value for this time field. Might be ignored by the client.
            placeholder(str): Placeholder text to be displayed in this input field.
            value(str): Initial value of this field.
            height(BlockElementHeight): Specifies the way the height of this container should 
                be calculated (stretch or auto).
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
        """
        self.id = id
        self.type = "Input.Time"
        self.max = max
        self.min = min
        self.placeholder = placeholder
        self.value = value
        self.height = height
        self.separator = separator
        self.spacing = spacing

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'id', 'type', 'max', 'min', 'placeholder', 'value',
                            'height', 'separator', 'spacing'
                        ])

class Toggle(Serializable):
    """Input field that lets a user toggle between multiple values."""
    def __init__(self, title, id, value=None,
                                  valueOff=None,
                                  valueOn=None,
                                  height=None,
                                  separator=None,
                                  spacing=None):
        """Create a new Toggle input.

        Args:
            title(str): Title of this toggle
            id(str): The id of this input.
            value(str): Initial selected value. Defaults to "false".
            valueOn(str): The value when toggle is on.
            valueOff(str): The value when toggle is off.
            height(BlockElementHeight): Specifies the way the height of this container should 
                be calculated (stretch or auto).
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
        """
        self.title = title
        self.type = "Input.Toggle"
        self.id = id
        self.value = value
        self.valueOff = valueOff
        self.valueOn = valueOn
        self.height = height
        self.separator = separator
        self.spacing = spacing

        super().__init__(serializable_properties=[],
                         simple_properties=[
                            'type', 'id', 'title', 'value', 'valueOff',
                            'valueOn', 'height', 'separator', 'spacing'
                        ])

class Choices(Serializable):
    """Input field that displays choices to the user.
    
    Note:
        The adaptive card documentation calls this input a "ChoiceSet"
    """
    def __init__(self, choices, id, isMultiSelect=None,
                                    style=None,
                                    value=None,
                                    height=None,
                                    separator=None,
                                    spacing=None):
        """Create a new Choices input. 

        Args:
            choices(list): List of Choice components to be displayed to the user.
            id(str): The id of this input.
            isMultiSelect(bool): If True, multiple choices can be selected.
            style(ChoiceInputStyle): The style of this choices input.
            value(str): The initial choice (or choices). Comma-seperated for multiple
                initial selections.
            height(BlockElementHeight): Specifies the way the height of this container should 
                be calculated (stretch or auto).
            separator(bool): Draw a separating line when set to true
            spacing(Spacing): Specify the spacing of this component
        """
        self.choices = choices
        self.type = "Input.ChoiceSet"
        self.id = id
        self.isMultiSelect = isMultiSelect
        self.style = style
        self.value = value
        self.height = height
        self.separator = separator
        self.spacing = spacing

        super().__init__(serializable_properties=['choices'],
                         simple_properties=[
                            'id', 'type', 'isMultiSelect', 'style', 'value',
                            'height', 'separator', 'spacing'
                         ])



# -*- coding: utf-8 -*-
"""Top-level package for PyAdaptiveCards.

Copyright (c) 2019 Cisco and/or its affiliates.

This software is licensed to you under the terms of the Cisco Sample
Code License, Version 1.1 (the "License"). You may obtain a copy of the
License at

               https://developer.cisco.com/docs/licenses

All use of the material herein must be in accordance with the terms of
the License. All rights not expressly granted by the License are
reserved. Unless required by applicable law or agreed to separately in
writing, software distributed under the License is distributed on an "AS
IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
or implied.
"""

__author__ = """Marcel Neidinger"""
__email__ = "mneiding@cisco.com"
__version__ = "0.1.0"
__copyright__ = "Copyright (c) 2019 Cisco and/or its affiliates."
__license__ = "Cisco Sample Code License, Version 1.1"



# -*- coding: utf-8 -*-
"""PyAdaptiveCards util functions.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

def set_if_not_none(property_name, property, export):
    if property is not None:
        export[property_name] = property.to_dict()

def check_type(obj, acceptable_types, is_list=False, may_be_none=False):
    """Object is an instance of one of the acceptable types or None.

    Args:
        obj: The object to be inspected.
        acceptable_types: A type or tuple of acceptable types.
        is_list(bool): Whether or not we expect a list of objects of acceptable
            type
        may_be_none(bool): Whether or not the object may be None.

    Raises:
        TypeError: If the object is None and may_be_none=False, or if the
            object is not an instance of one of the acceptable types.

    """
    error_message = None
    if not isinstance(acceptable_types, tuple):
        acceptable_types = (acceptable_types,)

    if may_be_none and obj is None:
        pass
    elif is_list:
        # Check that all objects in that list are of the required type
        if not isinstance(obj, list):
            error_message = (
                "We were expecting to receive a list of one of the following "
                "types: {types}{none}; but instead we received {o} which is a "
                "{o_type}.".format(
                    types=", ".join([repr(t.__name__) for t in acceptable_types]),
                    none="or 'None'" if may_be_none else "",
                    o=obj,
                    o_type=repr(type(obj).__name__)
                )
            )
        else:
            for o in obj:
                if not isinstance(o, acceptable_types):
                    error_message = (
                        "We were expecting to receive an instance of one of the following "
                        "types: {types}{none}; but instead we received {o} which is a "
                        "{o_type}.".format(
                            types=", ".join([repr(t.__name__) for t in acceptable_types]),
                            none="or 'None'" if may_be_none else "",
                            o=o,
                            o_type=repr(type(o).__name__)
                        )
                    )
    elif isinstance(obj, acceptable_types):
        pass
    else:
        # Object is something else.
        error_message = (
            "We were expecting to receive an instance of one of the following "
            "types: {types}{none}; but instead we received {o} which is a "
            "{o_type}.".format(
                types=", ".join([repr(t.__name__) for t in acceptable_types]),
                none="or 'None'" if may_be_none else "",
                o=obj,
                o_type=repr(type(obj).__name__)
            )
        )
    if error_message is not None:
        raise TypeError(error_message)



# -*- coding: utf-8 -*-
"""PyAdaptiveCards Card wrapper.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .abstract_components import Serializable
from .actions import OpenUrl, ShowCard, Submit

from .utils import check_type

class AdaptiveCard(Serializable):
    """AdaptiveCard class that represents a adaptive card python object.

    Note:
        Webex Teams currently supports version 1.1 of adaptive cards and thus
        only features from that release are supported in this abstraction.
    """
    def __init__(self, body=None,
                       actions=None,
                       selectAction=None,
                       fallbackText=None,
                       lang=None):
        """Creates a new adaptive card object.

        Args:
            body(list): The list of components and containers making up the
                body of this adaptive card.
            actions(list): The list of actions this adaptive card should contain
            selectAction(action): The action that should be invoked when this
                adaptive card is selected. Can be any action other then
                'ShowCard'
            fallbackText(str): The text that should be displayed on clients that
                can't render adaptive cards
            lang(str): The 2-letter ISO-639-1 language used in the card. This is
                used for localization of date/time functions

        """
        # Check types
        check_type(actions, (ShowCard, Submit, OpenUrl), True, True)
        check_type(selectAction, (Submit, OpenUrl), False, True)
        check_type(fallbackText, str, False, True)
        check_type(lang, str, False, True)

        # Set properties
        self.type = "AdaptiveCard"
        self.version = "1.1" # This is the version currently supported in Teams
        self.body = body
        self.actions = actions
        self.selectAction = selectAction
        self.fallbackText = fallbackText
        self.lang = lang

        super().__init__(serializable_properties=[
                            'body', 'actions', 'selectAction'
                         ],
                         simple_properties=[
                            'version', 'fallbackText', 'lang', 'type'
                         ])
    def to_dict(self):
        # We need to overwrite the to_dict method to add the $schema
        # property that can't be specified the normal way due to the
        # $ in the beginning
        ret = super().to_dict()
        ret["$schema"] = "http://adaptivecards.io/schemas/adaptive-card.json"

        return ret



# -*- coding: utf-8 -*-
"""PyAdaptiveCards Actions Wrapper.

Copyright (c) 2016-2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .abstract_components import Serializable

class OpenUrl(Serializable):
    """Open a external url when being invoked. """
    def __init__(self, url, title=None,
                            iconURL=None):
        """Create a new OpenUrl action. 

        Args:
            url(str): The URL that is opened upon being invoked.
            title(str): Label for the button or link that represents this action.
            iconURL(str): URL to the icon (currently not supported in Webex Teams)
        """
        self.type = "Action.OpenUrl"
        self.url = url
        self.title = title
        self.iconURL = iconURL

        super().__init__(serializable_properties=[],
                         simple_properties=['type', 'url', 'title', 'iconURL'])

class Submit(Serializable):
    """Gather input fields in the card and submit them."""
    def __init__(self, data=None,
                       title=None,
                       iconURL=None,
                       ):
        """Create a new Submit action.

        Args:
            data(dict): Initial data. In html forms this would be hidden fields.
            title(str): Label for the button or link that represents this action.
            iconURL(str): URL to the icon (currently not supported in Webex Teams).
        """
        self.type = "Action.Submit"
        self.data = data
        self.title = title
        self.iconURL = iconURL

        super().__init__(serializable_properties=[],
                         simple_properties=['data', 'title', 'iconURL', 'type'])

class ShowCard(Serializable):
    """Shows the specified adaptive card when this action/button is clicked."""
    def __init__(self, card=None,
                       title=None,
                       iconURL=None):
        """Create a new ShowCard action.

        Args:
            card(AdaptiveCard): The adaptive card that is shown.
            title(str): Label for the button or link that represents this action.
            iconURL(str): URL to the icon (currently not supported in Webex Teams).
        """
        self.type = "Action.ShowCard"
        self.card = card
        self.title = title
        self.iconURL = iconURL

        super().__init__(serializable_properties=['card'],
                         simple_properties=['title', 'type', 'iconURL'])
